package
{
	import flash.events.EventDispatcher;
	import flash.utils.setTimeout;
	import flash.utils.clearTimeout;
	import flash.utils.getTimer;
	import TimerEvent;
	
	public class Timer extends EventDispatcher
	{
		/**
		* Initializes a new instance of the Timer class, and sets the Interval property to the specified time period.
		* @param	interval
		* The time, in milliseconds, between events.
		* The default is 100 milliseconds.
		* @throws	Error
		* if the interval specified is negative or not a finite number
		*/
		function Timer(interval:int = 100)
		{
			if(interval <= 0 || !isFinite(interval)) throw new Error("Invalid value for interval.");
			
			m_interval = interval;
			m_autoReset = true;
			m_count = 0;
			m_enabled = false;
		}
		
		/**
		* Gets or sets a value indicating whether the Timer should raise the Elapsed event each
		* time the specified interval elapses or only after the first time it elapses.
		* @param	value
		* true if the Timer should raise the Elapsed event each time the interval elapses;
		* false if it should raise the Elapsed event only once, after the first time the interval elapses.
		* The default is true.
		*/
		public function set AutoReset(value:Boolean):void
		{
			m_autoReset = value;
		}
		public function get AutoReset():Boolean
		{
			return m_autoReset;
		}
		
		/**
		* The total number of times the timer has fired since it started at zero.
		* If the timer has been reset, only the fires since the reset are counted.
		*/
		public function get Count():int
		{
			return m_count;
		}
		
		/**
		* Gets or sets a value indicating whether the Timer should raise the Elapsed event.
		* <p>
		* Setting Enabled to true is the same as calling Start, while setting Enabled to false
		* is the same as calling Stop.
		* </p>
		* <p>
		* If the interval is set after the Timer has started, the count is reset. For example,
		* if you set the interval to 5 seconds and then set the Enabled property to true, the
		* count starts at the time Enabled is set. If you reset the interval to 10 seconds when
		* count is 3 seconds, the Elapsed event is raised for the first time 13 seconds after
		* Enabled was set to true.
		* <p>
		* @param	value
		* true if the Timer should raise the Elapsed event; otherwise, false.
		* The default is false.
		*/
		public function set Enabled(value:Boolean):void
		{
			value ? Start() : Stop();
		}
		public function get Enabled():Boolean
		{
			return m_enabled;
		}
		
		/**
		* Gets or sets the interval at which to raise the Elapsed event.
		* <p>
		* If the interval is set after the Timer has started, the count is reset. For example,
		* if you set the interval to 5 seconds and then set the Enabled property to true,
		* the count starts at the time Enabled is set. If you reset the interval to 10 seconds
		* when count is 3 seconds, the Elapsed event is raised for the first time 13 seconds
		* after Enabled was set to true.
		* </p>
		* <p>
		* If Enabled is set to true and AutoReset is set to false, the Timer raises the Elapsed
		* event only once, the first time the interval elapses.
		* </p>
		* @param	value
		* The time, in milliseconds, between raisings of the Elapsed event.
		* The default is 100 milliseconds.
		*/
		public function set Interval(value:int):void
		{
			Stop();
			m_interval = value;
			Start();
		}
		public function get Interval():int
		{
			return m_interval;
		}
		
		/**
		* Starts raising the Elapsed event by setting Enabled to true.
		* <p>
		* If Enabled is set to true and AutoReset is set to false, the Timer raises the Elapsed
		* event only once, the first time he interval elapses. When Enabled is true and AutoReset
		* is true, the Timer continues to raise the Elapsed event on the specified interval.
		* </p>
		* <p>
		* You can also start timing by setting Enabled to true.
		* </p>
		*/
		public function Start():void
		{
			if(m_enabled)
				clearTimeout(m_intervalId);
			m_enabled = true;
			
			m_count = 0;
			m_startTime = getTimer();
			m_intervalId = setTimeout(OnTimeout, m_interval);
		}
		
		/**
		* Stops raising the Elapsed event by setting Enabled to false.
		* <p>
		* You can also stop timing by setting Enabled to false.
		* </p>
		*/
		public function Stop():void
		{
			if(m_enabled)
				clearTimeout(m_intervalId);
			m_enabled = false;
		}
		
		//{ private
		private var m_interval:int;
		private var m_autoReset:Boolean;
		private var m_enabled:Boolean;
		
		private var m_intervalId:int;
		private var m_startTime:int;
		private var m_count:int;
		
		private function OnTimeout():void
		{
			while(m_enabled && (getTimer()-m_startTime) >= m_interval*(m_count+1))
			{
				m_count++;
				dispatchEvent( new TimerEvent(TimerEvent.ELAPSED, getTimer()-m_startTime-m_interval*m_count) );
			}
			
			if(m_enabled && m_autoReset)
			{
				// The delay is caught up with the next setTimeout
				var delay:int = getTimer()-m_startTime-m_interval*m_count
				m_intervalId = setTimeout(OnTimeout, m_interval-delay);
			}
			else
			{
				m_enabled = false;
			}
		}
		//}
	}
}
